@@ -1,4 +1,4 @@
-from datetime import datetime, timedelta, timezone
+from datetime import datetime
 from getpass import getuser
 from itertools import groupby
 import errno
@@ -15,9 +15,9 @@ import time
 from io import BytesIO
 from attic import xattr
 from attic.platform import acl_get, acl_set
-from attic.chunker import chunkify
+from attic.chunker import Chunker
 from attic.hashindex import ChunkIndex
-from attic.helpers import Error, uid2user, user2uid, gid2group, group2gid, \
+from attic.helpers import parse_timestamp, Error, uid2user, user2uid, gid2group, group2gid, \
     Manifest, Statistics, decode_dict, st_mtime_ns, make_path_safe, StableDict, int_to_bigint, bigint_to_int
 
 ITEMS_BUFFER = 1024 * 1024
@@ -26,10 +26,17 @@ WINDOW_SIZE = 0xfff
 CHUNK_MASK = 0xffff
 
 utime_supports_fd = os.utime in getattr(os, 'supports_fd', {})
+utime_supports_follow_symlinks = os.utime in getattr(os, 'supports_follow_symlinks', {})
 has_mtime_ns = sys.version >= '3.3'
 has_lchmod = hasattr(os, 'lchmod')
 has_lchflags = hasattr(os, 'lchflags')
 
+# Python <= 3.2 raises OSError instead of PermissionError (See #164)
+try:
+    PermissionError = PermissionError
+except NameError:
+    PermissionError = OSError
+
 
 class DownloadPipeline:
 
@@ -64,6 +71,7 @@ class ChunkBuffer:
         self.packer = msgpack.Packer(unicode_errors='surrogateescape')
         self.chunks = []
         self.key = key
+        self.chunker = Chunker(WINDOW_SIZE, CHUNK_MASK, CHUNK_MIN, self.key.chunk_seed)
 
     def add(self, item):
         self.buffer.write(self.packer.pack(StableDict(item)))
@@ -77,10 +85,10 @@ class ChunkBuffer:
         if self.buffer.tell() == 0:
             return
         self.buffer.seek(0)
-        chunks = list(bytes(s) for s in chunkify(self.buffer, WINDOW_SIZE, CHUNK_MASK, CHUNK_MIN, self.key.chunk_seed))
+        chunks = list(bytes(s) for s in self.chunker.chunkify(self.buffer))
         self.buffer.seek(0)
         self.buffer.truncate(0)
-        # Leave the last parital chunk in the buffer unless flush is True
+        # Leave the last partial chunk in the buffer unless flush is True
         end = None if flush or len(chunks) == 1 else -1
         for chunk in chunks[:end]:
             self.chunks.append(self.write_chunk(chunk))
@@ -111,6 +119,10 @@ class Archive:
     class AlreadyExists(Error):
         """Archive {} already exists"""
 
+    class IncompatibleFilesystemEncodingError(Error):
+        """Failed to encode filename "{}" into file system encoding "{}". Consider configuring the LANG environment variable."""
+
+
     def __init__(self, repository, key, manifest, name, cache=None, create=False,
                  checkpoint_interval=300, numeric_owner=False):
         self.cwd = os.getcwd()
@@ -123,9 +135,10 @@ class Archive:
         self.name = name
         self.checkpoint_interval = checkpoint_interval
         self.numeric_owner = numeric_owner
-        self.items_buffer = CacheChunkBuffer(self.cache, self.key, self.stats)
         self.pipeline = DownloadPipeline(self.repository, self.key)
         if create:
+            self.items_buffer = CacheChunkBuffer(self.cache, self.key, self.stats)
+            self.chunker = Chunker(WINDOW_SIZE, CHUNK_MASK, CHUNK_MIN, self.key.chunk_seed)
             if name in manifest.archives:
                 raise self.AlreadyExists(name)
             self.last_checkpoint = time.time()
@@ -154,8 +167,7 @@ class Archive:
     @property
     def ts(self):
         """Timestamp of archive creation in UTC"""
-        t, f = self.metadata[b'time'].split('.', 1)
-        return datetime.strptime(t, '%Y-%m-%dT%H:%M:%S').replace(tzinfo=timezone.utc) + timedelta(seconds=float('.' + f))
+        return parse_timestamp(self.metadata[b'time'])
 
     def __repr__(self):
         return 'Archive(%r)' % self.name
@@ -199,9 +211,9 @@ class Archive:
 
     def calc_stats(self, cache):
         def add(id):
-            count, size, csize = self.cache.chunks[id]
+            count, size, csize = cache.chunks[id]
             stats.update(size, csize, count == 1)
-            self.cache.chunks[id] = count - 1, size, csize
+            cache.chunks[id] = count - 1, size, csize
         def add_file_chunks(chunks):
             for id, _, _ in chunks:
                 add(id)
@@ -239,6 +251,8 @@ class Archive:
                 os.rmdir(path)
             else:
                 os.unlink(path)
+        except UnicodeEncodeError:
+            raise self.IncompatibleFilesystemEncodingError(path, sys.getfilesystemencoding())
         except OSError:
             pass
         mode = item[b'mode']
@@ -314,7 +328,7 @@ class Archive:
         mtime = bigint_to_int(item[b'mtime'])
         if fd and utime_supports_fd:  # Python >= 3.3
             os.utime(fd, None, ns=(mtime, mtime))
-        elif utime_supports_fd:  # Python >= 3.3
+        elif utime_supports_follow_symlinks:  # Python >= 3.3
             os.utime(path, None, ns=(mtime, mtime), follow_symlinks=False)
         elif not symlink:
             os.utime(path, (mtime / 1e9, mtime / 1e9))
@@ -353,7 +367,7 @@ class Archive:
             item[b'xattrs'] = StableDict(xattrs)
         if has_lchflags and st.st_flags:
             item[b'bsdflags'] = st.st_flags
-        item[b'acl'] = acl_get(path, item, self.numeric_owner)
+        acl_get(path, item, st, self.numeric_owner)
         return item
 
     def process_item(self, path, st):
@@ -396,9 +410,9 @@ class Archive:
                 chunks = [cache.chunk_incref(id_, self.stats) for id_ in ids]
         # Only chunkify the file if needed
         if chunks is None:
-            with open(path, 'rb') as fd:
+            with Archive._open_rb(path, st) as fd:
                 chunks = []
-                for chunk in chunkify(fd, WINDOW_SIZE, CHUNK_MASK, CHUNK_MIN, self.key.chunk_seed):
+                for chunk in self.chunker.chunkify(fd):
                     chunks.append(cache.add_chunk(self.key.id_hash(chunk), chunk, self.stats))
             cache.memorize_file(path_hash, st, [c[0] for c in chunks])
         item = {b'path': safe_path, b'chunks': chunks}
@@ -411,6 +425,45 @@ class Archive:
         for name, info in manifest.archives.items():
             yield Archive(repository, key, manifest, name, cache=cache)
 
+    @staticmethod
+    def _open_rb(path, st):
+        flags_noatime = None
+        euid = None
+
+        def open_simple(p, s):
+            return open(p, 'rb')
+
+        def open_noatime_if_owner(p, s):
+            if s.st_uid == euid:
+                return os.fdopen(os.open(p, flags_noatime), 'rb')
+            else:
+                return open(p, 'rb')
+
+        def open_noatime(p, s):
+            try:
+                fd = os.open(p, flags_noatime)
+            except PermissionError:
+                # Was this EPERM due to the O_NOATIME flag?
+                fo = open(p, 'rb')
+                # Yes, it was -- otherwise the above line would have thrown
+                # another exception.
+                euid = os.geteuid()
+                # So in future, let's check whether the file is owned by us
+                # before attempting to use O_NOATIME.
+                Archive._open_rb = open_noatime_if_owner
+                return fo
+            return os.fdopen(fd, 'rb')
+
+        o_noatime = getattr(os, 'O_NOATIME', None)
+        if o_noatime is not None:
+            flags_noatime = os.O_RDONLY | getattr(os, 'O_BINARY', 0) | o_noatime
+            # Always use O_NOATIME version.
+            Archive._open_rb = open_noatime
+        else:
+            # Always use non-O_NOATIME version.
+            Archive._open_rb = open_simple
+        return Archive._open_rb(path, st)
+
 
 class RobustUnpacker():
     """A restartable/robust version of the streaming msgpack unpacker
@@ -503,7 +556,7 @@ class ArchiveChecker:
         # Explicity set the initial hash table capacity to avoid performance issues
         # due to hash table "resonance"
         capacity = int(len(self.repository) * 1.2)
-        self.chunks = ChunkIndex.create(os.path.join(self.tmpdir, 'chunks').encode('utf-8'), capacity=capacity)
+        self.chunks = ChunkIndex(capacity)
         marker = None
         while True:
             result = self.repository.list(limit=10000, marker=marker)
