@@ -62,6 +62,7 @@ class Archiver:
         manifest.key = key
         manifest.write()
         repository.commit()
+        Cache(repository, key, manifest, warn_if_unencrypted=False)
         return self.exit_code
 
     def do_check(self, args):
@@ -186,19 +187,30 @@ Type "Yes I am sure" if you understand this and want to continue.\n""")
 
     def do_extract(self, args):
         """Extract archive contents"""
+        # be restrictive when restoring files, restore permissions later
+        if sys.getfilesystemencoding() == 'ascii':
+            print('Warning: File system encoding is "ascii", extracting non-ascii filenames will not be supported.')
+        os.umask(0o077)
         repository = self.open_repository(args.archive)
         manifest, key = Manifest.load(repository)
         archive = Archive(repository, key, manifest, args.archive.archive,
                           numeric_owner=args.numeric_owner)
         patterns = adjust_patterns(args.paths, args.excludes)
+        dry_run = args.dry_run
+        strip_components = args.strip_components
         dirs = []
         for item in archive.iter_items(lambda item: not exclude_path(item[b'path'], patterns), preload=True):
+            orig_path = item[b'path']
+            if strip_components:
+                item[b'path'] = os.sep.join(orig_path.split(os.sep)[strip_components:])
+                if not item[b'path']:
+                    continue
             if not args.dry_run:
                 while dirs and not item[b'path'].startswith(dirs[-1][b'path']):
                     archive.extract_item(dirs.pop(-1))
-            self.print_verbose(remove_surrogates(item[b'path']))
+            self.print_verbose(remove_surrogates(orig_path))
             try:
-                if args.dry_run:
+                if dry_run:
                     archive.extract_item(item, dry_run=True)
                 else:
                     if stat.S_ISDIR(item[b'mode']):
@@ -207,7 +219,7 @@ Type "Yes I am sure" if you understand this and want to continue.\n""")
                     else:
                         archive.extract_item(item)
             except IOError as e:
-                self.print_error('%s: %s', remove_surrogates(item[b'path']), e)
+                self.print_error('%s: %s', remove_surrogates(orig_path), e)
 
         if not args.dry_run:
             while dirs:
@@ -302,7 +314,7 @@ Type "Yes I am sure" if you understand this and want to continue.\n""")
         print('Username:', archive.metadata[b'username'])
         print('Time: %s' % to_localtime(archive.ts).strftime('%c'))
         print('Command line:', remove_surrogates(' '.join(archive.metadata[b'cmdline'])))
-        print('Number of files: %d' % archive.stats.nfiles)
+        print('Number of files: %d' % stats.nfiles)
         stats.print_('This archive:', cache)
         return self.exit_code
 
@@ -572,6 +584,9 @@ Type "Yes I am sure" if you understand this and want to continue.\n""")
         subparser.add_argument('--numeric-owner', dest='numeric_owner',
                                action='store_true', default=False,
                                help='only obey numeric user and group identifiers')
+        subparser.add_argument('--strip-components', dest='strip_components',
+                               type=int, default=0, metavar='NUMBER',
+                               help='Remove the specified number of leading path elements. Pathnames with fewer elements will be silently skipped.')
         subparser.add_argument('archive', metavar='ARCHIVE',
                                type=location_validator(archive=True),
                                help='archive to extract')
