@@ -5,7 +5,6 @@ import msgpack
 import os
 import pwd
 import re
-import stat
 import sys
 import time
 from datetime import datetime, timezone, timedelta
@@ -73,10 +72,10 @@ class UpgradableLock:
 
 def check_extension_modules():
     import attic.platform
-    if (attic.hashindex.API_VERSION != 1 or
-        attic.chunker.API_VERSION != 1 or
+    if (attic.hashindex.API_VERSION != 2 or
+        attic.chunker.API_VERSION != 2 or
         attic.crypto.API_VERSION != 2 or
-        attic.platform.API_VERSION != 1):
+        attic.platform.API_VERSION != 2):
         raise ExtensionModuleError
 
 
@@ -185,6 +184,14 @@ def to_localtime(ts):
     return datetime(*time.localtime((ts - datetime(1970, 1, 1, tzinfo=timezone.utc)).total_seconds())[:6])
 
 
+def parse_timestamp(timestamp):
+    """Parse a ISO 8601 timestamp string"""
+    if '.' in timestamp:  # microseconds might not be pressent
+        return datetime.strptime(timestamp, '%Y-%m-%dT%H:%M:%S.%f').replace(tzinfo=timezone.utc)
+    else:
+        return datetime.strptime(timestamp, '%Y-%m-%dT%H:%M:%S').replace(tzinfo=timezone.utc)
+
+
 def update_excludes(args):
     """Merge exclude patterns from files with those on command line.
     Empty lines and lines starting with '#' are ignored, but whitespace
@@ -277,17 +284,6 @@ def is_cachedir(path):
     return False
 
 
-def walk_path(path, skip_inodes=None):
-    st = os.lstat(path)
-    if skip_inodes and (st.st_ino, st.st_dev) in skip_inodes:
-        return
-    yield path, st
-    if stat.S_ISDIR(st.st_mode):
-        for f in os.listdir(path):
-            for x in walk_path(os.path.join(path, f), skip_inodes):
-                yield x
-
-
 def format_time(t):
     """Format datetime suitable for fixed length list output
     """
@@ -412,11 +408,11 @@ class Location:
     proto = user = host = port = path = archive = None
     ssh_re = re.compile(r'(?P<proto>ssh)://(?:(?P<user>[^@]+)@)?'
                         r'(?P<host>[^:/#]+)(?::(?P<port>\d+))?'
-                        r'(?P<path>[^:]+)(?:::(?P<archive>.+))?')
+                        r'(?P<path>[^:]+)(?:::(?P<archive>.+))?$')
     file_re = re.compile(r'(?P<proto>file)://'
-                         r'(?P<path>[^:]+)(?:::(?P<archive>.+))?')
+                         r'(?P<path>[^:]+)(?:::(?P<archive>.+))?$')
     scp_re = re.compile(r'((?:(?P<user>[^@]+)@)?(?P<host>[^:/]+):)?'
-                        r'(?P<path>[^:]+)(?:::(?P<archive>.+))?')
+                        r'(?P<path>[^:]+)(?:::(?P<archive>.+))?$')
 
     def __init__(self, text):
         self.orig = text
@@ -468,6 +464,21 @@ class Location:
     def __repr__(self):
         return "Location(%s)" % self
 
+    def canonical_path(self):
+        if self.proto == 'file':
+            return self.path
+        else:
+            if self.path and self.path.startswith('~'):
+                path = '/' + self.path
+            elif self.path and not self.path.startswith('/'):
+                path = '/~/' + self.path
+            else:
+                path = self.path
+            return 'ssh://{}{}{}{}'.format('{}@'.format(self.user) if self.user else '',
+                                                        self.host,
+                                                        ':{}'.format(self.port) if self.port else '',
+                                                        path)
+
 
 def location_validator(archive=None):
     def validator(text):
@@ -509,7 +520,7 @@ def remove_surrogates(s, errors='replace'):
     return s.encode('utf-8', errors).decode('utf-8')
 
 
-_safe_re = re.compile('^((..)?/+)+')
+_safe_re = re.compile(r'^((\.\.)?/+)+')
 
 
 def make_path_safe(path):
@@ -577,3 +588,4 @@ def int_to_bigint(value):
     if value.bit_length() > 63:
         return value.to_bytes((value.bit_length() + 9) // 8, 'little', signed=True)
     return value
+
