@@ -8,11 +8,20 @@ from hashlib import sha256
 import zlib
 
 from attic.crypto import pbkdf2_sha256, get_random_bytes, AES, bytes_to_long, long_to_bytes, bytes_to_int, num_aes_blocks
-from attic.helpers import IntegrityError, get_keys_dir
+from attic.helpers import IntegrityError, get_keys_dir, Error
 
 PREFIX = b'\0' * 8
 
 
+class UnsupportedPayloadError(Error):
+    """Unsupported payload type {}. A newer version is required to access this repository.
+    """
+
+class KeyfileNotFoundError(Error):
+    """No key file for repository {} found in {}.
+    """
+
+
 class HMAC(hmac.HMAC):
     """Workaround a bug in Python < 3.4 Where HMAC does not accept memoryviews
     """
@@ -37,7 +46,7 @@ def key_factory(repository, manifest_data):
     elif manifest_data[0] == PlaintextKey.TYPE:
         return PlaintextKey.detect(repository, manifest_data)
     else:
-        raise Exception('Unkown Key type %d' % ord(manifest_data[0]))
+        raise UnsupportedPayloadError(manifest_data[0])
 
 
 class KeyBase(object):
@@ -217,7 +226,7 @@ class KeyfileKey(AESKeyBase):
                 line = fd.readline().strip()
                 if line and line.startswith(cls.FILE_ID) and line[10:] == id:
                     return filename
-        raise Exception('Key file for repository with ID %s not found' % id)
+        raise KeyfileNotFoundError(repository._location.canonical_path(), get_keys_dir())
 
     def load(self, filename, passphrase):
         with open(filename, 'r') as fd:
