@@ -44,6 +44,9 @@ class Repository(object):
     class CheckNeeded(Error):
         '''Inconsistency detected. Please run "attic check {}"'''
 
+    class ObjectNotFound(Error):
+        """Object with key {} not found in repository {}"""
+
     def __init__(self, path, create=False, exclusive=False):
         self.path = path
         self.io = None
@@ -128,24 +131,22 @@ class Repository(object):
         self.write_index()
         self.rollback()
 
-    def get_read_only_index(self, transaction_id):
+    def open_index(self, transaction_id):
         if transaction_id is None:
-            return {}
-        return NSIndex((os.path.join(self.path, 'index.%d') % transaction_id).encode('utf-8'), readonly=True)
+            return NSIndex()
+        return NSIndex.read((os.path.join(self.path, 'index.%d') % transaction_id).encode('utf-8'))
 
-    def get_index(self, transaction_id, do_cleanup=True):
+    def prepare_txn(self, transaction_id, do_cleanup=True):
         self._active_txn = True
         self.lock.upgrade()
+        if not self.index:
+            self.index = self.open_index(transaction_id)
         if transaction_id is None:
-            self.index = NSIndex.create(os.path.join(self.path, 'index.tmp').encode('utf-8'))
             self.segments = {}
             self.compact = set()
         else:
             if do_cleanup:
                 self.io.cleanup(transaction_id)
-            shutil.copy(os.path.join(self.path, 'index.%d' % transaction_id),
-                        os.path.join(self.path, 'index.tmp'))
-            self.index = NSIndex(os.path.join(self.path, 'index.tmp').encode('utf-8'))
             hints = read_msgpack(os.path.join(self.path, 'hints.%d' % transaction_id))
             if hints[b'version'] != 1:
                 raise ValueError('Unknown hints file version: %d' % hints['version'])
@@ -158,7 +159,7 @@ class Repository(object):
                  b'compact': list(self.compact)}
         transaction_id = self.io.get_segments_transaction_id()
         write_msgpack(os.path.join(self.path, 'hints.%d' % transaction_id), hints)
-        self.index.flush()
+        self.index.write(os.path.join(self.path, 'index.tmp'))
         os.rename(os.path.join(self.path, 'index.tmp'),
                   os.path.join(self.path, 'index.%d' % transaction_id))
         # Remove old indices
@@ -199,7 +200,7 @@ class Repository(object):
         self.compact = set()
 
     def replay_segments(self, index_transaction_id, segments_transaction_id):
-        self.get_index(index_transaction_id, do_cleanup=False)
+        self.prepare_txn(index_transaction_id, do_cleanup=False)
         for segment, filename in self.io.segment_iterator():
             if index_transaction_id is not None and segment <= index_transaction_id:
                 continue
@@ -248,7 +249,7 @@ class Repository(object):
         assert not self._active_txn
         try:
             transaction_id = self.get_transaction_id()
-            current_index = self.get_read_only_index(transaction_id)
+            current_index = self.open_index(transaction_id)
         except Exception:
             transaction_id = self.io.get_segments_transaction_id()
             current_index = None
@@ -259,7 +260,7 @@ class Repository(object):
         if repair:
             self.io.cleanup(transaction_id)
         segments_transaction_id = self.io.get_segments_transaction_id()
-        self.get_index(None)
+        self.prepare_txn(None)
         for segment, filename in self.io.segment_iterator():
             if segment > transaction_id:
                 continue
@@ -310,8 +311,6 @@ class Repository(object):
         if repair:
             self.compact_segments()
             self.write_index()
-        else:
-            os.unlink(os.path.join(self.path, 'index.tmp'))
         self.rollback()
         return not error_found or repair
 
@@ -323,22 +322,22 @@ class Repository(object):
 
     def __len__(self):
         if not self.index:
-            self.index = self.get_read_only_index(self.get_transaction_id())
+            self.index = self.open_index(self.get_transaction_id())
         return len(self.index)
 
     def list(self, limit=None, marker=None):
         if not self.index:
-            self.index = self.get_read_only_index(self.get_transaction_id())
+            self.index = self.open_index(self.get_transaction_id())
         return [id_ for id_, _ in islice(self.index.iteritems(marker=marker), limit)]
 
     def get(self, id_):
         if not self.index:
-            self.index = self.get_read_only_index(self.get_transaction_id())
+            self.index = self.open_index(self.get_transaction_id())
         try:
             segment, offset = self.index[id_]
             return self.io.read(segment, offset, id_)
         except KeyError:
-            raise self.DoesNotExist(self.path)
+            raise self.ObjectNotFound(id_, self.path)
 
     def get_many(self, ids, is_preloaded=False):
         for id_ in ids:
@@ -346,7 +345,7 @@ class Repository(object):
 
     def put(self, id, data, wait=True):
         if not self._active_txn:
-            self.get_index(self.get_transaction_id())
+            self.prepare_txn(self.get_transaction_id())
         try:
             segment, _ = self.index[id]
             self.segments[segment] -= 1
@@ -363,11 +362,11 @@ class Repository(object):
 
     def delete(self, id, wait=True):
         if not self._active_txn:
-            self.get_index(self.get_transaction_id())
+            self.prepare_txn(self.get_transaction_id())
         try:
             segment, offset = self.index.pop(id)
         except KeyError:
-            raise self.DoesNotExist(self.path)
+            raise self.ObjectNotFound(id, self.path)
         self.segments[segment] -= 1
         self.compact.add(segment)
         segment = self.io.write_delete(id)
@@ -409,12 +408,14 @@ class LoggedIO(object):
         self.fds = None  # Just to make sure we're disabled
 
     def segment_iterator(self, reverse=False):
-        for dirpath, dirs, filenames in os.walk(os.path.join(self.path, 'data')):
-            dirs.sort(key=int, reverse=reverse)
-            filenames = sorted((filename for filename in filenames if filename.isdigit()), key=int, reverse=reverse)
-            for filename in filenames:
-                yield int(filename), os.path.join(dirpath, filename)
-
+        data_path = os.path.join(self.path, 'data')
+        dirs = sorted((dir for dir in os.listdir(data_path) if dir.isdigit()), key=int, reverse=reverse)
+        for dir in dirs:
+            filenames = os.listdir(os.path.join(data_path, dir))
+            sorted_filenames = sorted((filename for filename in filenames
+                                       if filename.isdigit()), key=int, reverse=reverse)
+            for filename in sorted_filenames:
+                yield int(filename), os.path.join(data_path, dir, filename)
 
     def get_latest_segment(self):
         for segment, filename in self.segment_iterator(reverse=True):
