@@ -21,10 +21,12 @@ class XattrTestCase(AtticTestCase):
         self.assert_equal(listxattr(self.symlink), [])
         setxattr(self.tmpfile.name, 'user.foo', b'bar')
         setxattr(self.tmpfile.fileno(), 'user.bar', b'foo')
-        self.assert_equal(set(listxattr(self.tmpfile.name)), set(['user.foo', 'user.bar']))
-        self.assert_equal(set(listxattr(self.tmpfile.fileno())), set(['user.foo', 'user.bar']))
-        self.assert_equal(set(listxattr(self.symlink)), set(['user.foo', 'user.bar']))
+        setxattr(self.tmpfile.name, 'user.empty', None)
+        self.assert_equal(set(listxattr(self.tmpfile.name)), set(['user.foo', 'user.bar', 'user.empty']))
+        self.assert_equal(set(listxattr(self.tmpfile.fileno())), set(['user.foo', 'user.bar', 'user.empty']))
+        self.assert_equal(set(listxattr(self.symlink)), set(['user.foo', 'user.bar', 'user.empty']))
         self.assert_equal(listxattr(self.symlink, follow_symlinks=False), [])
         self.assert_equal(getxattr(self.tmpfile.name, 'user.foo'), b'bar')
         self.assert_equal(getxattr(self.tmpfile.fileno(), 'user.foo'), b'bar')
         self.assert_equal(getxattr(self.symlink, 'user.foo'), b'bar')
+        self.assert_equal(getxattr(self.tmpfile.name, 'user.empty'), None)
